#ifndef BUSPIRATESCOPE_H_INCLUDED
#define BUSPIRATESCOPE_H_INCLUDED

#include "..\miniscope\device\MiniscopeDeviceCapabilities.h"
#include <vector>
#include <string>
#include <windows.h>
#include "singleton.h"

class BusPirateScope : public CSingleton<BusPirateScope>
{
private:
    friend class CSingleton<BusPirateScope>;
    BusPirateScope();
    ~BusPirateScope();

	bool bRunning;
	bool bStopRequest;

    unsigned int sample_count;  // counter for samples sent to user
    enum E_TRIGGER_MODE mode;
    struct S_SCOPE_SETTINGS settings, upcoming_settings;
    bool bManualTriggerRequest;
	unsigned int lastTriggerDistance;	///< "distance" in samples from last trigger used to start capture in auto mode
    unsigned int frame_position;	    ///< current data buffer position (0 when just sent trigger signal)
    /** \brief Test for trigger condition
    	\param len number of samples added to trigger queue recently
    	\return index of first sample after triggering, < 0 if not triggered
    */
    int TestTrigger(short* samples, int len);
    HANDLE Thread;
    HANDLE hCommDev;
    DCB dcb;

    typedef void (*CallbackLog)(char*);
    typedef void (*CallbackData)(short*, int);
    typedef void (*CallbackConnect)(int, char*);
    typedef void (*CallbackTrigger)(void);
    void log(const char* const lpData, ...);
    void OnDataReceive(short samples[], unsigned int count);
    void ExitContinuousSampling(void);
    friend DWORD WINAPI ThreadProc(LPVOID data);

public:

    int Connect(void);
    int Disconnect(void);
    bool connected;

    CallbackLog callbackLog;
    CallbackData callbackData;
    CallbackConnect callbackConnect;
    CallbackTrigger callbackTrigger;
    void ConnectCb(int state, char* msg)
    {
        if (callbackConnect)
            callbackConnect(state, msg);
    }
    /** \brief Request manual triggering
    	\return 0 on success
    */
    int ManualTrigger(void);
    int Run(bool state);
    int SetTriggerMode(enum E_TRIGGER_MODE);
    int SetBufferSize (int size)
    {
        upcoming_settings.iBufferSize = size;
        log("Buffer (upcoming) size set to %d samples.", upcoming_settings.iBufferSize);
        return 0;
    }
    int SetTriggerType(enum E_TRIGGER_TYPE);
    int SetTriggerLevel(int level);
};

#define busPirateScope BusPirateScope::Instance()

#endif // BUSPIRATESCOPE_H_INCLUDED
